unit apperase;

{******************************************************************************
Unit apperase declares the invisible TAppEraser component which can be used to
delete any wished files (in one directory) after the application ends
(EVEN THE APPLICATION-FILE AND ITS DIRECTORY itself !)
This can be useful for Installation/Deinstallation-Tools to delete itselves
after their work (and desired dll's or something like that in the same
directory).

This stuff is freeware (not PD) and copyrighted by MARKUS STEPHANY at
MirBir.St@Saargate.de / MirBir.St@T-Online.de. there are no limitations to
free-/shareware-authors, commercial developers please contact me before you
use this component.

there are no guaranties and warranties and there is no possibility to make me
responsable for any misbehaviour, data-loss or anything else what my happen
if you use AppErase, but you are welcome to write me if there are any questions,
suggestions, bug-reports...

revision 1.0, october 09, 1997

******************************************************************************}

interface

uses
  Windows, SysUtils, Classes,filectrl;

type
  TAppEraser = class(tcomponent)
  private
    { Private-Deklarationen }
    ffiles : tstrings;
    fpath  : string;
    factive,fdelp: boolean;
    procedure setfiles (val:tstrings);
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    constructor create (aowner:tcomponent); override;
    destructor destroy ; override;

    // execute the deletion by yourself if autoactive is set to false
    // if the result is false, the execution failed
    // else the batch has been started (and will be never ended, if the
    // program runs from a CD or an other write-protect medium)
    function execute:boolean;
  published
    { Published-Deklarationen }

    // if this is true, the \/ files are to be deleted on the termination of the prog
    property AutoActive : boolean read factive write factive default false;

    // these are the files to be deleted, by default the program's filename
    property Files  : tstrings read ffiles write setfiles;

    //this is the path for the files to be deleted, by default the prog's path
    property Path   : string read fpath write fpath;

    // if this is true the app's path will be deleted after termination if there are no
    // other files than specified in the Files-property
    property DelPathIfEmpty : boolean read fdelp write fdelp default false;
  end;

procedure Register;

implementation

// helper functions

function getshort(pt,fn:string):string;
         // returns the short filename if any, else ''
var buf : pointer;
begin
     result := '';
     if pt = '' then exit;
     if fn = '' then exit;
     if lstrcmpi('*appname*',pchar(fn)) = 0 then fn := extractfilename(paramstr(0));
     fn := copy(pt+extractfilename(fn),1,max_path);
     if not fileexists(fn) then exit;
     buf := stralloc(max_path+2);
     strpcopy(buf,fn);
     getshortpathname(buf,buf,max_path+1);
     result := extractfilename(strpas(buf));
     strdispose(buf);
end;

function getrndfilename : string;
         // retrieve a random filename
var ct : integer;
const allowch :string = '0123456789abcdefghijklmnopqrstuvwxyz';
begin
     randomize;
     setlength(result,8);
     for ct := 1 to 8 do
         result[ct]:=allowch[random(length(allowch))+1];
end;

// TAppEraser implementation

constructor tapperaser.create (aowner:tcomponent);
begin
     inherited create(aowner);
     factive := false;
     fdelp := false;
     ffiles := tstringlist.create;
     if csdesigning in componentstate then ffiles.add ('*AppName*');
     fpath := '*AppPath*';
end;

destructor tapperaser.destroy ;
begin
     if factive and (not (csdesigning in componentstate)) then execute;
     ffiles.free;
     inherited destroy;
end;

procedure tapperaser.setfiles (val:tstrings);
begin
     ffiles.assign(val);
end;

function tapperaser.execute;
var batch,short,lastpath,firstpath,ftp : string;
    batchnum,ct : integer;
    f : system.text;
    happened : boolean;
    buf : pchar;
begin
     result := false;
     // fpath is the rootpath, trim to correct form
     if fpath='' then exit;
     if lstrcmpi('*apppath*',pchar(fpath))=0 then
        fpath := extractfilepath(paramstr(0));
     if fpath[length(fpath)] <> '\' then fpath := fpath+'\';
     ftp := copy(fpath,1,length(fpath)-1);
     if not directoryexists(fpath) then
        if not directoryexists(ftp) then exit;
     if (ffiles= nil) or (ffiles.count = 0) then exit;

     // and get the short path name
     buf := stralloc(max_path+2);
     strpcopy(buf,ftp);
     if getshortpathname(buf,buf,max_path+1)= 0 then begin
        strdispose(buf);
        exit;
     end;
     ftp := strpas(buf);
     strdispose(buf);
     if copy(ftp,length(ftp),1) = '\' then ftp := copy(ftp,1,length(ftp)-1);
     // get the path names

     firstpath := extractfilepath(ftp);
     lastpath := extractfilename(ftp);

     // first get a temporary name for the batch-file
     repeat
           batch := getrndfilename;
     until not fileexists(firstpath+batch+'.bat');
     batch := batch+'.bat';

     // create the textfile
     try
        assignfile(f,firstpath+batch);
        rewrite(f);
        happened := false;
        writeln(f,'cd .\'+lastpath);

        // now create entries for each file in the ffiles-specs
        for ct := 0 to ffiles.count -1 do begin

            // get the short filename of each file
            short := getshort(fpath,ffiles[ct]);
            if short <> '' then begin
               // allow deleting, if not on a write-protect medium
               filesetattr(fpath+extractfilename(ffiles[ct]),0);

               // label to return if del not succeeded
               writeln(f,':label'+inttostr(ct));
               // try to delete the file
               writeln(f,'del '+short);
               // repeat if not deleted
               writeln(f,'if exist '+short+' goto label'+inttostr(ct));

               happened := true;
            end;
        end;

        if not happened then begin
           // no file to erase
           closefile(f);
           deletefile(fpath+batch);
           exit;
        end;

        // return to the master-path
        writeln(f,'cd ..');

        // if we should delete the path
        if fdelp then
           writeln(f,'rmdir .\'+lastpath);
	// rmdir removes a path only if it is empty, therefor : no fear ;)

        // append the strings for deleting the batch itself
        writeln(f,':label'+inttostr(ct));
        // try to delete the file
        writeln(f,'del '+batch);
        // repeat if not deleted
        writeln(f,'if exist '+batch+' goto label'+inttostr(ct));
        closefile(f);

        // now set the correct directory and start erasing
        chdir(firstpath);
        result := winexec(pchar(batch),sw_hide)>31;
     except
           result := false;
     end;

end;

procedure Register;
begin
  RegisterComponents('Samples', [TAppEraser]);
end;

end.
